@@ -1,6 +1,6 @@
 #!/usr/bin/ruby
 
-require 'vips'
+require "vips"
 
 # this makes vips keep a list of all active objects
 # Vips::leak_set true
@@ -15,7 +15,7 @@ if ARGV.length < 2
   raise "usage: #{$PROGRAM_NAME}: input-file output-file"
 end
 
-im = Vips::Image.new_from_file ARGV[0], :access => :sequential
+im = Vips::Image.new_from_file ARGV[0], access: :sequential
 
 im *= [1, 2, 1]
 
@@ -23,8 +23,8 @@ im *= [1, 2, 1]
 # make it ourselves
 # if you are OK with scale=1, you can just pass the array directly to .conv()
 mask = Vips::Image.new_from_array [[-1, -1, -1],
-                                   [-1, 16, -1],
-                                   [-1, -1, -1]], 8
+  [-1, 16, -1],
+  [-1, -1, -1]], 8
 im = im.conv mask
 
 im.write_to_file ARGV[1]
