@@ -4,8 +4,8 @@
 # Author::    John Cupitt  (mailto:jcupitt@gmail.com)
 # License::   MIT
 
-require 'ffi'
-require 'forwardable'
+require "ffi"
+require "forwardable"
 
 module GObject
   # we have a number of things we need to inherit in different ways:
@@ -37,13 +37,15 @@ module GObject
     def_instance_delegators :@struct, :[], :to_ptr
     def_single_delegators :ffi_struct, :ptr
 
+    attr_reader :references
+
     # the layout of the GObject struct
     module GObjectLayout
       def self.included base
         base.class_eval do
           layout :g_type_instance, :pointer,
-                 :ref_count, :uint,
-                 :qdata, :pointer
+            :ref_count, :uint,
+            :qdata, :pointer
         end
       end
     end
@@ -56,7 +58,7 @@ module GObject
         # GLib::logger.debug("GObject::GObject::ManagedStruct.release") {
         #     "unreffing #{ptr}"
         # }
-        ::GObject::g_object_unref ptr
+        ::GObject.g_object_unref ptr
       end
     end
 
@@ -72,6 +74,7 @@ module GObject
     # need the unref
     def initialize ptr
       # GLib::logger.debug("GObject::GObject.initialize") {"ptr = #{ptr}"}
+      @ptr = ptr
       @struct = ffi_managed_struct.new ptr
 
       # sometimes we need to keep refs across C calls ... hide them here
@@ -83,9 +86,13 @@ module GObject
       self.class.ffi_struct
     end
 
+    # get the pointer we were built from ... #to_ptr gets the pointer after we
+    # have wrapped it up with an auto unref
+    attr_reader :ptr
+
     class << self
       def ffi_struct
-        self.const_get :Struct
+        const_get :Struct
       end
     end
 
@@ -96,7 +103,7 @@ module GObject
 
     class << self
       def ffi_managed_struct
-        self.const_get :ManagedStruct
+        const_get :ManagedStruct
       end
     end
   end
@@ -104,10 +111,10 @@ module GObject
   class GParamSpec < FFI::Struct
     # the first few public fields
     layout :g_type_instance, :pointer,
-           :name, :string,
-           :flags, :uint,
-           :value_type, :GType,
-           :owner_type, :GType
+      :name, :string,
+      :flags, :uint,
+      :value_type, :GType,
+      :owner_type, :GType
   end
 
   class GParamSpecPtr < FFI::Struct
