@@ -48,8 +48,9 @@ module Vips
     end
 
     def initialize name
-      ptr = Vips::vips_interpolate_new name
-      raise Vips::Error if ptr == nil
+      name = name.to_s if name.is_a? Symbol
+      ptr = Vips.vips_interpolate_new name
+      raise Vips::Error if ptr.nil?
 
       super ptr
     end
